# Generated by Django 3.1.3 on 2020-11-21 21:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0019_auto_20201115_1844'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='server',
            options={'verbose_name': 'Guild', 'verbose_name_plural': 'Guilds'},
        ),
    ]
