# Generated by Django 3.1.2 on 2020-10-23 20:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0008_amongusgame_last_tracking_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='amongusgame',
            name='tracker_connected',
        ),
    ]
