# Generated by Django 3.1.2 on 2020-10-31 15:15

import discordbot.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0009_remove_amongusgame_tracker_connected'),
    ]

    operations = [
        migrations.CreateModel(
            name='VierGewinntGame',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('width', models.PositiveSmallIntegerField(default=7, verbose_name='Width')),
                ('height', models.PositiveSmallIntegerField(default=6, verbose_name='Height')),
                ('game', models.JSONField(default=discordbot.models.VIERGEWINNT_DEFAULT_GAME, verbose_name='Game')),
                ('finished', models.BooleanField(default=False, verbose_name='Finished')),
                ('current_player', models.PositiveSmallIntegerField(default=1, verbose_name='Current player')),
                ('player_1_id', models.CharField(max_length=32, verbose_name='Player 1 ID')),
                ('player_2_id', models.CharField(max_length=32, verbose_name='Player 2 ID')),
                ('winner_id', models.CharField(default=None, max_length=32, null=True, verbose_name='Winner ID')),
                ('channel_id', models.CharField(max_length=32, verbose_name='Channel ID')),
                ('message_id', models.CharField(max_length=32, verbose_name='Message ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, verbose_name='Time created')),
                ('time_edited', models.DateTimeField(auto_now=True, verbose_name='Time edited')),
            ],
            options={
                'verbose_name': 'VierGewinnt Game',
                'verbose_name_plural': 'VierGewinnt Games',
            },
        ),
    ]
