# Generated by Django 3.1.2 on 2020-10-22 07:03

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0002_auto_20200701_1655'),
    ]

    operations = [
        migrations.CreateModel(
            name='AmongUsGame',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('api_key', models.UUIDField(default=uuid.uuid4, verbose_name='API Key')),
                ('voice_channel_id', models.CharField(max_length=20, verbose_name='Voice channel ID')),
                ('text_channel_id', models.CharField(max_length=20, verbose_name='Text channel ID')),
                ('code', models.CharField(blank=True, default='', max_length=6, verbose_name='Gamecode')),
                ('last_edited', models.DateTimeField(auto_now=True, verbose_name='Last edited')),
                ('tracker_connected', models.DateTimeField(default=False, verbose_name='Tracker connected')),
                ('state_ingame', models.BooleanField(default=False, verbose_name='In game')),
                ('state_meeting', models.BooleanField(default=False, verbose_name='Meeting ongoing')),
                ('p_red_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Red's name")),
                ('p_red_alive', models.BooleanField(default=True, verbose_name="Red's alive")),
                ('p_red_exists', models.BooleanField(default=True, verbose_name='Red exists')),
                ('p_purple_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Purple's name")),
                ('p_purple_alive', models.BooleanField(default=True, verbose_name="Purple's alive")),
                ('p_purple_exists', models.BooleanField(default=True, verbose_name='Purple exists')),
                ('p_orange_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Orange's name")),
                ('p_orange_alive', models.BooleanField(default=True, verbose_name="Orange's alive")),
                ('p_orange_exists', models.BooleanField(default=True, verbose_name='Orange exists')),
                ('p_yellow_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Yellow's name")),
                ('p_yellow_alive', models.BooleanField(default=True, verbose_name="Yellow's alive")),
                ('p_yellow_exists', models.BooleanField(default=True, verbose_name='Yellow exists')),
                ('p_white_name', models.CharField(blank=True, default='', max_length=50, verbose_name="White's name")),
                ('p_white_alive', models.BooleanField(default=True, verbose_name="White's alive")),
                ('p_white_exists', models.BooleanField(default=True, verbose_name='White exists')),
                ('p_black_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Black's name")),
                ('p_black_alive', models.BooleanField(default=True, verbose_name="Black's alive")),
                ('p_black_exists', models.BooleanField(default=True, verbose_name='Black exists')),
                ('p_green_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Green's name")),
                ('p_green_alive', models.BooleanField(default=True, verbose_name="Green's alive")),
                ('p_green_exists', models.BooleanField(default=True, verbose_name='Green exists')),
                ('p_lime_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Lime's name")),
                ('p_lime_alive', models.BooleanField(default=True, verbose_name="Lime's alive")),
                ('p_lime_exists', models.BooleanField(default=True, verbose_name='Lime exists')),
                ('p_blue_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Blue's name")),
                ('p_blue_alive', models.BooleanField(default=True, verbose_name="Blue's alive")),
                ('p_blue_exists', models.BooleanField(default=True, verbose_name='Blue exists')),
                ('p_cyan_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Cyan's name")),
                ('p_cyan_alive', models.BooleanField(default=True, verbose_name="Cyan's alive")),
                ('p_cyan_exists', models.BooleanField(default=True, verbose_name='Cyan exists')),
                ('p_brown_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Brown's name")),
                ('p_brown_alive', models.BooleanField(default=True, verbose_name="Brown's alive")),
                ('p_brown_exists', models.BooleanField(default=True, verbose_name='Brown exists')),
                ('p_pink_name', models.CharField(blank=True, default='', max_length=50, verbose_name="Pink's name")),
                ('p_pink_alive', models.BooleanField(default=True, verbose_name="Pink's alive")),
                ('p_pink_exists', models.BooleanField(default=True, verbose_name='Pink exists')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.user')),
                ('guild', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='discordbot.server')),
            ],
            options={
                'verbose_name': 'AmongUs Game',
                'verbose_name_plural': 'AmongUs Games',
                'unique_together': {('guild', 'creator')},
            },
        ),
    ]
