# Generated by Django 3.0.4 on 2020-07-01 14:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('discordbot', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='server',
            old_name='discordid',
            new_name='id',
        ),
        migrations.RenameField(
            model_name='user',
            old_name='discordid',
            new_name='id',
        ),
    ]
